/* Written by Tibor 'Igor2' Palinkas. This file is in the public domain. */

/* Derived from: 01_main_sc_stdin.c
   Compile: cc -o 03_main_callarg 03_main_callarg.c -lm
   Execute: ./03_main_callarg < hello.fawk
   Features:
    - loads a script from a stdin
    - execute the main part with ARGV[] set to "hey" and 42
    - no "include" support
*/

/*** libfawk configuration ***/
typedef double fawk_num_t;
typedef long fawk_refco_t;
#define FAWK_API static

/* uncomment this to disable fawk_print() -> cuts off stdio dependency */
/*#define FAWK_DISABLE_FAWK_PRINT*/

/* include the single-c, all language variant */
#include "../../../single_c/libfawk_sc_all/libfawk_sc_all.c"

/*** glue ***/
FAWK_API void libfawk_error(fawk_ctx_t *ctx, const char *str, const char *loc_fn, long loc_line, long loc_col)
{
	fprintf(stderr, "Error: %s\n", str);
}

static int getch1(fawk_ctx_t *ctx, fawk_src_t *src)
{
	return getchar();
}

int main(int argc, char *argv[])
{
	FILE *fsc;
	fawk_ctx_t ctx;

	/* set up the script context */
	fawk_init(&ctx);
	ctx.parser.get_char = getch1;

	/* parse; replace fawk_parse_fawk() with fawk_parse_fpas() or
	   fawk_parse_fbas() for the other languages */
	if (fawk_parse_fawk(&ctx) != 0) {
		fawk_uninit(&ctx);
		return 1;
	}

	/* call main() - call1 prepares the stack */
	if (fawk_call1(&ctx, "main") == 0) {
		fawk_cell_t res, *sargv, *v;
		fawk_execret_t er;

		/* main() has only one argument: an array ARGV: */
		sargv = fawk_push_alloc(&ctx); /* push the first argument, get the cell referencing that argument */
		fawk_array_init(&ctx, sargv);  /* initialize it as an array */

		/* resolve the index [0] of the array and fill it in with a string */
		v = fawk_array_resolve_c(&ctx, 1, sargv, FAWK_NUM, (fawk_num_t)(0), FAWK_NIL);
		v->type = FAWK_STR;
		v->data.str = fawk_str_new_from_literal(&ctx, "hey", -1);

		/* resolve the index [1] of the array and fill it in with a number */
		v = fawk_array_resolve_c(&ctx, 1, sargv, FAWK_NUM, (fawk_num_t)(1), FAWK_NIL);
		v->type = FAWK_NUM;
		v->data.num = 42;

		if (fawk_call2(&ctx, 1) == 0) { /* call2: perform call, with 1 argument on stack */
			er = fawk_execute(&ctx, -1); /* execute the VM; will return once the function has finished */
			if (er == FAWK_ER_FIN) { /* read and free the return value of script's main() */
				fawk_pop(&ctx, &res);
				printf("\nResult: ");
				fawk_dump_cell(&res, 1);
				printf("\n");
				fawk_cell_free(&ctx, &res);
			}
		}
		else
			fprintf(stderr, "Failed to call main() [#2]\n");
	}
	else
		fprintf(stderr, "Failed to call main() [#1]\n");

	fawk_uninit(&ctx);

	return 0;
}

