/*
libfawk - A function-only AWK dialect

Copyright (c) 2017..2020 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#include <stdlib.h>
#include <string.h>
#include "libfawk.h"
#include "execute.h"
#include "symtab.h"
#include "str.h"

static fawk_code_t *grow(fawk_ctx_t *ctx)
{
	if (ctx->code.used >= ctx->code.alloced) {
		fawk_code_t *c;
		ctx->code.alloced += 1024;
		if ((c = fawk_realloc(ctx, ctx->code.code, ctx->code.alloced * sizeof(fawk_code_t))) == NULL) { ctx->code.alloced = 0; return NULL; }
		ctx->code.code = c;
	}
	ctx->code.code[ctx->code.used].line = ctx->parser.isp->line+1;
	return &ctx->code.code[ctx->code.used++];
}

FAWK_API void fawkc_addi(fawk_ctx_t *ctx, fawk_ins_t ins)
{
	fawk_code_t *i = grow(ctx); if (i == NULL) { return; }
	i->type = FAWKC_INS;
	i->data.ins = ins;
}

FAWK_API void fawkc_addcs(fawk_ctx_t *ctx, const char *s)
{
	fawk_code_t *i = grow(ctx); if (i == NULL) { return; }
	i->type = FAWKC_CSTR;
	i->data.cstr = s;
}

FAWK_API void fawkc_adds(fawk_ctx_t *ctx, const char *s)
{
	fawk_code_t *i = grow(ctx); if (i == NULL) { return; }
	i->data.str = fawk_str_new_from_literal(ctx, s, -1);
	i->type = (i->data.str == NULL) ? FAWKC_NUM : FAWKC_STR;
}

FAWK_API void fawkc_addnum(fawk_ctx_t *ctx, fawk_num_t num)
{
	fawk_code_t *i = grow(ctx); if (i == NULL) { return; }
	i->type = FAWKC_NUM;
	i->data.num = num;
}

FAWK_API int fawkc_addsymref(fawk_ctx_t *ctx, const char *name, int isarr, int stack_from)
{
	fawk_code_t *i;
	fawk_cell_t *c;
	int n, offs;

	for(n = stack_from, offs = -ctx->fp-1+stack_from; n < ctx->fp; n++,offs++) {
		c = fawk_peek(ctx, n);
		fawk_assert(c->type == FAWK_STR);
		if (strcmp(name, c->data.str->str) == 0) {
			i = grow(ctx); if (i == NULL) { return -1; }
			i->type = FAWKC_SYMREF;
			i->data.symref = fawk_calloc(ctx, sizeof(fawk_symref_t), 1); if (i->data.symref == NULL) return -1;
			i->data.symref->is_local = 1;
			i->data.symref->ref.local = offs-1;
			return 0;
		}
	}

	/* global variable */
	if ((c = fawk_symtab_regvar(ctx, name, isarr ? FAWK_ARRAY : FAWK_SCALAR)) == NULL)
		return -1;

	i = grow(ctx); if (i == NULL) { return -1; }
	i->type = FAWKC_SYMREF;
	i->data.symref = fawk_calloc(ctx, sizeof(fawk_symref_t), 1); if (i->data.symref == NULL) return -1;
/*	i->data.symref->is_local = 0; - this was set in fawk_calloc() above */
	i->data.symref->ref.global = c;

	return 0;
}
